/*******************************************************************************
 * Copyright (c) 2014 Karlsruhe Institute of Technology, Germany
 *                    Technical University Darmstadt, Germany
 *                    Chalmers University of Technology, Sweden
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Technical University Darmstadt - initial API and implementation and/or initial documentation
 *******************************************************************************/

package org.key_project.sed.key.ui.test.testcase.swtbot;

import org.eclipse.swtbot.eclipse.finder.widgets.SWTBotView;
import org.eclipse.swtbot.swt.finder.widgets.SWTBotTree;
import org.junit.Test;
import org.key_project.util.test.util.SWTBotTabbedPropertyList;
import org.key_project.util.test.util.TestUtilsUtil;

/**
 * Tests the property page tab "Loop Invariant".
 * @author Martin Hentschel
 */
public class SWTBotLoopInvariantTabTest extends AbstractSWTBotKeYPropertyTabTest {
   /**
    * Tests the shown values and the existence of tab "KeY".
    */
   @Test
   public void testValuesAndTabExistence() throws Exception {
      doAllNodeTypesTest("SWTBotLoopInvariantTabTest_testValuesAndTabExistence", createAllNodeTypesSteps());
   }
   
   /**
    * Creates the test steps to execute.
    * @return The created test steps.
    */
   public static ITestSteps createAllNodeTypesSteps() {
      return new ITestSteps() {
         @Override
         public void assertThread(SWTBotTree debugTree, SWTBotView propertiesView, SWTBotTabbedPropertyList tabs) throws Exception {
            assertFalse(tabs.selectTabItem("Loop Invariant"));
         }
         
         @Override
         public void assertStatement(SWTBotTree debugTree, SWTBotView propertiesView, SWTBotTabbedPropertyList tabs) throws Exception {
            assertFalse(tabs.selectTabItem("Loop Invariant"));
         }
         
         @Override
         public void assertDebugTarget(SWTBotTree debugTree, SWTBotView propertiesView, SWTBotTabbedPropertyList tabs) throws Exception {
            assertFalse(tabs.hasTabItem("Loop Invariant"));
         }

         @Override
         public void assertMethodReturn(SWTBotTree debugTree, SWTBotView propertiesView, SWTBotTabbedPropertyList tabs) throws Exception {
            assertFalse(tabs.selectTabItem("Loop Invariant"));
         }

         @Override
         public void assertLaunch(SWTBotTree debugTree, SWTBotView propertiesView, SWTBotTabbedPropertyList tabs) throws Exception {
            assertFalse(tabs.hasTabItem("Loop Invariant"));
         }

         @Override
         public void assertTermination(SWTBotTree debugTree, SWTBotView propertiesView, SWTBotTabbedPropertyList tabs) throws Exception {
            assertFalse(tabs.selectTabItem("Loop Invariant"));
         }

         @Override
         public void assertMethodContract(SWTBotTree debugTree, SWTBotView propertiesView, SWTBotTabbedPropertyList tabs) throws Exception {
            assertFalse(tabs.selectTabItem("Loop Invariant"));
         }

         @Override
         public void assertLoopInvariant(SWTBotTree debugTree, SWTBotView propertiesView, SWTBotTabbedPropertyList tabs) throws Exception {
            assertTrue(tabs.selectTabItem("Loop Invariant"));
            TestUtilsUtil.waitForJobs();
            assertEquals(" true\n" +
            		       "==>\n" +
                         "   (i >= 0)<<F(\"8.0\")>>\n" +
            				 " & (i <= 1)<<F(\"9.0\")>>\n" +
                         " & wellFormed(heap)<<F(\"10.0\")>>", propertiesView.bot().styledText(0).getText());
         }

         @Override
         public void assertLoopBodyTermination(SWTBotTree debugTree, SWTBotView propertiesView, SWTBotTabbedPropertyList tabs) throws Exception {
            assertTrue(tabs.selectTabItem("Loop Invariant"));
            TestUtilsUtil.waitForJobs();
            assertEquals(" true\n" +
                         "==>\n" +
                         "   (i >= 0)<<F(\"11.0\")>>\n" +
                         " & (i <= 1)<<F(\"12.0\")>>\n" +
                         " & (\\forall Field f;\n" +
                         "      (\\forall java.lang.Object o;\n" +
                         "         (o.f@heapBefore_LOOP = o.f)<<F(\"13.0\")>>)<<F(\"14.0\")>>)<<F(\"15.0\")>>\n" +
                         " & prec(2 + i * -1, variant)<<F(\"16.0\")>>", propertiesView.bot().styledText(0).getText());
         }
      };
   }
}